"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deposit = void 0;
const web3_js_1 = require("@solana/web3.js");
const pda_1 = require("../../utils/pda");
const shared_1 = require("../../constants/shared");
const associated_token_1 = require("@project-serum/associated-token");
const spl_token_1 = require("@solana/spl-token");
class Deposit {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, campaign, pool, poolNode, authority, amount) {
        const deposit = new web3_js_1.Keypair();
        const tempTokenAccount = new web3_js_1.Keypair();
        const [depositAuthority] = (0, pda_1.deriveDepositAuthority)(deposit.publicKey, client.programId);
        const [clearing] = (0, pda_1.derivePublicClearingAddress)(client.cypherProgramId);
        const [cypherAccount, accountBump] = (0, pda_1.deriveAccountAddress)(depositAuthority, 0, client.cypherProgramId);
        const [cypherSubAccount, subAccountBump] = (0, pda_1.deriveSubAccountAddress)(cypherAccount, 0, client.cypherProgramId);
        const fundingAccount = await (0, associated_token_1.getAssociatedTokenAddress)(authority, campaign.state.assetMint);
        const ix = await client.methods
            .createDeposit(accountBump, subAccountBump, amount)
            .accountsStrict({
            campaign: campaign.address,
            deposit: deposit.publicKey,
            cacheAccount: shared_1.CONFIGS[client.cluster].CACHE,
            clearing,
            cypherAccount,
            cypherSubAccount,
            depositAuthority,
            fundingAccount,
            tempTokenAccount: tempTokenAccount.publicKey,
            assetMint: campaign.state.assetMint,
            pool: pool.address,
            poolNode: poolNode.address,
            poolNodeVault: poolNode.vaultAddress(),
            payer: authority,
            signer: authority,
            cypherProgram: client.cypherProgramId,
            tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
            rent: web3_js_1.SYSVAR_RENT_PUBKEY,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            deposit: deposit.publicKey,
            ixs: [ix],
            signers: [deposit, tempTokenAccount]
        };
    }
    async end(client, campaign, pool, poolNode, authority) {
        const tempTokenAccount = new web3_js_1.Keypair();
        const campaignRewardVault = campaign.rewardVaultAddress();
        const campaignRewardVaultAuthority = campaign.rewardVaultAuthorityAddress();
        const [depositAuthority] = (0, pda_1.deriveDepositAuthority)(this.address, client.programId);
        const [clearing] = (0, pda_1.derivePublicClearingAddress)(client.cypherProgramId);
        const [cypherAccount] = (0, pda_1.deriveAccountAddress)(depositAuthority, 0, client.cypherProgramId);
        const [cypherSubAccount] = (0, pda_1.deriveSubAccountAddress)(cypherAccount, 0, client.cypherProgramId);
        const assetTokenAccount = await (0, associated_token_1.getAssociatedTokenAddress)(authority, campaign.state.assetMint);
        const rewardTokenAccount = await (0, associated_token_1.getAssociatedTokenAddress)(authority, campaign.state.rewardMint);
        const ix = await client.methods
            .endDeposit()
            .accountsStrict({
            campaign: campaign.address,
            campaignRewardVault,
            campaignRewardVaultAuthority,
            deposit: this.address,
            cacheAccount: shared_1.CONFIGS[client.cluster].CACHE,
            clearing,
            cypherAccount,
            cypherSubAccount,
            depositAuthority,
            tempTokenAccount: tempTokenAccount.publicKey,
            assetTokenAccount,
            assetMint: campaign.state.assetMint,
            pool: pool.address,
            poolNode: poolNode.address,
            poolNodeVault: poolNode.vaultAddress(),
            poolNodeVaultSigner: poolNode.vaultSignerAddress(),
            rewardTokenAccount,
            rewardMint: campaign.state.rewardMint,
            payer: authority,
            signer: authority,
            cypherProgram: client.cypherProgramId,
            tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
            associatedTokenProgram: spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID,
            rent: web3_js_1.SYSVAR_RENT_PUBKEY,
            systemProgram: web3_js_1.SystemProgram.programId
        })
            .instruction();
        return {
            deposit: this.address,
            ixs: [ix],
            signers: [tempTokenAccount]
        };
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = (await client.accounts.deposit.fetch(address));
        return new Deposit(client, address, state, onStateUpdateHandler, errorCallback);
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('Deposit', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
}
exports.Deposit = Deposit;
//# sourceMappingURL=deposit.js.map